/*
 * Purpose: private wxWindows mail transport implementation
 * Author:  Frank Bu
 * Created: 2002
 */

#ifndef SMPT_H
#define SMPT_H

#include <wx/wx.h>
#include <wx/protocol/protocol.h>

#include "email.h"
#include "cmdprot.h"


/**
 * Public available interface for classes who wants to be notified asynchronly about
 * SMTP events. Such classes can use this class as a base class and implement
 * the methods of this class for receiving and evaluating some events.
 * \sa For an example see WinoDialog.
 */
class wxSMTPListener
{
public:
	/**
	 * Will be called on errors. Not implemented.
	 * \param errorCode TODO: What type of error code should we use?
	 */
	virtual void OnSocketError(int errorCode) {};

	/**
	 * Will be called if the SMTP server has acknowledged the RCPT command, which
	 * is the case, if a recipient was added.
	 * \param address The recipient who was added.
	 * \param errorCode 1, if successfully added, 0 otherwise. TODO: perhaps a error enumeration type would be better.
	 */
	virtual void OnRecipientAdded(const wxString& address, int errorCode) {};

	/**
	 * Will be called if the SMTP server has acknowledged the DATA command, which
	 * is the case, if the email text was sent. TODO: Currently this is only sent, if the
	 * email was successfully sent.
	 * \param address The recipient who was added.
	 * \param errorCode 1, if successfully added, 0 otherwise. TODO: perhaps a error enumeration type would be better.
	 */
	virtual void OnDataSent(int errorCode) {};
};

class MailState;

/**
 * Implementation of the sendmail class for SMTP email sending.
 */
class wxSMTP : public wxCmdlineProtocol //, public wxEvtHandler
{
public:
    wxSMTP(wxSMTPListener* pListener = NULL);

	~wxSMTP();

	/**
	 * Called by Sendmail::Send().
	 * \sa Sendmail::Send()
	 */
	void Send(wxEmailMessage* pMessage);

	void EvaluateLine(const wxString& line);
	void OnConnect(wxSocketEvent& event);

	// MailStateContext interface

	/**
	 * Changes the internal state.
	 * \param pMailState The new state.
	 * \sa MailState.
	 */
	void ChangeState(const MailState& mailState);

	/**
	 * Is called by the SendMailFromState, if the SMTP server has acknowledged.
	 * This method sends then the next recipient, until all recipients have been
	 * sent. Then the StartDataState will be set.
	 */
	void SendNextRecipient();

	/**
	 * Is called by RcptListState if the SMTP server denied the current recipient.
	 */
	void OnRecipientError();

	/**
	 * Is called by RcptListState if the SMTP server accepts the current recipient.
	 */
	void OnRecipientSuccess();

	/**
	 * Is called by DataState if the data was sent successfully.
	 */
	void OnDataSuccess();

	/**
	 * Is called by HeloState to indicate that the from command can be sent.
	 */
	void SendFrom();

	/**
	 * Is called by StartDataState to indicate that the DATA command can be sent.
	 */
	void SendData();

private:
	const MailState* m_pMailState;
	wxSMTPListener* m_pListener;
	wxEmailMessage* m_pMessage;

	// current recipient and counts
	size_t m_count;
	wxString m_currentRecipient;

	// all recipients
	wxArrayString m_recipients;
};

#endif
