/*
 * Purpose: private wxWindows helper classes for SMTP
 * Author:  Frank Bu
 * Created: 2002
 */

#ifndef STATES_H
#define STATES_H

#include <wx/wx.h>
#include <wx/socket.h>

#include "smtp.h"

/**
 * Base class for all mail states. Every mail state is only instantiated once
 * and has no attributes (flyweight pattern? I don't know the name). The only
 * reason for the state classes is not to have a big switch-case block.
 */
class MailState
{
public:
	virtual void onConnect(wxSMTP& context, wxSocketEvent& event) const {}
	virtual void onResponse(wxSMTP& context, int smtpCode) const {}
};

/**
 * This state is the initial state, when no server connection is made.
 */
class InitialState : public MailState
{
public:
	/**
	 * Switches to ConnectedState, if successfully connected.
	 * \sa ConnectedState.
	 */
	void onConnect(wxSMTP& context, wxSocketEvent& event) const ;
};


/**
 * This state is active, if the server connection is established.
 */
class ConnectedState : public MailState
{
public:
	/**
	 * Switches to HeloState, if welcome message is received.
	 * \sa HeloState.
	 */
	void onResponse(wxSMTP& context, int smtpCode) const ;
};


/**
 * This state is active, when the helo message was sent.
 */
class HeloState : public MailState
{
public:
	/**
	 * Switches to SendMailFromState, if the helo message was acknowledged.
	 * \sa SendMailFromState.
	 */
	void onResponse(wxSMTP& context, int smtpCode) const ;
};


/**
 * This state is active, when the from message was sent.
 */
class SendMailFromState : public MailState
{
public:
	/**
	 * Switches to RcptListState, if the from message was acknowledged.
	 * \sa RcptListState.
	 */
	void onResponse(wxSMTP& context, int smtpCode) const ;
};


/**
 * This state is active as long as there are additional recipients
 * and no error occured (TODO: error-handling not implemented).
 */
class RcptListState : public MailState
{
public:
	/**
	 * SendmailImpl wwitches to StartDataState, if there are no more recipients.
	 * \sa StartDataState.
	 */
	void onResponse(wxSMTP& context, int smtpCode) const ;
};


/**
 * This state is active, when the DATA command was sent.
 */
class StartDataState : public MailState
{
public:
	/**
	 * Switches to DataState, if the DATA command was acknowledged.
	 * \sa DataState.
	 */
	void onResponse(wxSMTP& context, int smtpCode) const ;
};


/**
 * This state is active, when the data was sent.
 */
class DataState : public MailState
{
public:
	/**
	 * Acknowledges by calling MailContext::OnDataSuccess, if the data sent was acknowledged.
	 * \sa SendmailImpl::OnDataSuccess().
	 */
	void onResponse(wxSMTP& context, int smtpCode) const ;
};

extern const InitialState g_initialState;
extern const ConnectedState g_connectedState;
extern const HeloState g_heloState;
extern const SendMailFromState g_sendMailFromState;
extern const RcptListState g_rcptListState;
extern const StartDataState g_startDataState;
extern const DataState g_dataState;

#endif
